% ident_via_contsid.m
% H. Garnier
% 11/01/2023
% Program which estimates the parameters of a first-order transfer function
% model from positive and then negative steps
% sent to the QUBE-servo 2

% Download, if needed, the CONTSID toolbox at
% www.cran.univ-lorraine.fr/contsid
% Add the folder CONTSID7_4 to the path of Matlab (Set Path in the menu bar)

clear;
close all;
clc

load data_step_Qube
data0=data_step;

t=data0(1,:)';
y0=data0(2,:)';
initial_speed=y0(3,1);
y=y0-initial_speed;   % Removal of the possibly non-zero initial angular speed 
u=data0(3,:)';        % for the simulation
Ts=t(2)-t(1); % sampling time
data=iddata(y,u,Ts);

idplot(data)
set(gca,'FontSize',14,'FontName','helvetica');
grid
disp('press a key to continue')
pause

% Estimate the 2 parameters (K:gain, T:time-constant) of a 
% first-order process model by using 
% the optimal iterative Instrumental Variable method SRIVC
% implemented in PROCSRIVC

model_type = idproc('P1'); % First-order model without delay
M1=procsrivc(data,model_type) % 
K=M1.Kp;
T=M1.Tp1;

yhat=compare(data,M1);
yhat=yhat.y;


% Computation of the FIT criteion which is the percentage of the 
% measured output that is explained by the model (see formula below)
% Perfect Model if FIT=100 %
FIT = 100 * (1-norm(y-yhat)/norm(y-mean(y)));

figure
plot(t,y+initial_speed,t,yhat+initial_speed,'linewidth',2)
xlabel('time (s)')
legend('Measured angular speed','Simulated angular speed','location','Northeast')
title(['Fit between the model and measured responses: ',num2str(FIT,'% 10.1f'),' %'])
set(gca,'FontSize',14,'FontName','helvetica');
grid
shg